##' Set tables for auxiliary variables \code{D}
##' 
##' Function for setting the tables on all nodes \code{D_r_a} for
##' mixture \code{r} in the network corresponding to one marker, using
##' peak heights and a set of parameters. 
##' 
##' @details
##' The function sets conditional probabilities for auxiliary variables \code{D_r_a},
##' which are used for obtaining probabilities of a peak
##' falling above resp. below the detection threshold under the current
##' model. 
##' The conditional probability tables are defined using the gamma c.d.f. as
##' \deqn{P(\texttt{D[a]} = \texttt{TRUE} | n_{ia}, n_{i,a+1}, i = 1,\ldots, k) = G(\texttt{C}).
##' }{P(D[a] = TRUE | n_{ia}, n_{i,a+1}, i = 1,\ldots, k) = G(C).}
##' @export
##' 
##' @param domain A \code{hugin.domain} modelling one marker
##' @param rho rho
##' @param xi xi
##' @param eta eta
##' @param phi phi
##' @param C Detection threshold
##' @param n.unknown Number of unknown contributors
##' @param n_K Allelecounts for known contributors
##' @param D Names for binary nodes
##' @param gets_stutter Does the allele receive stutter?
##' @param can_stutter Can the allele stutter?
##' @param stutter.from From which allele does the stutter come?
##' @return invisibly NULL
##' @seealso For further details see \code{\link{setCPT.O}}.
setCPT.D <- function(domain, rho, xi, eta, phi, C, n.unknown,
                     n_K, D, gets_stutter, can_stutter, stutter.from){

  ## Combinations of allele counts for one allele all contributors.
  n_U <- expand.grid(rep(list(0:2), n.unknown), KEEP.OUT.ATTRS = FALSE)
  
  one.allele <- function(a){

    ## Find shape parameters 
    ## Contribution from allele a
    n_a <- as.matrix(merge(n_U, n_K[a,])) ## = n_U if n_K is NULL
    shapes <- rho * (1 - xi*can_stutter[a]) * (n_a %*% phi)

    if (gets_stutter[a]){
      ## Adding contributions from stutter from allele b
      b <- stutter.from[a]
      n_b <- as.matrix(merge(n_U, n_K[b,])) ## using n_Ua = n_Ub
      newshapes <- rho * xi*can_stutter[b] * (n_b %*% phi)
      shapes <- outer(shapes, newshapes, "+")
    }
    shapes <- as.vector(shapes)

    ## This works even when shape = 0.
    unseen.probs <- pgamma(C, shape = shapes, scale = eta)
    seen.probs <- pgamma(C, shape = shapes, scale = eta, lower.tail = FALSE)

    ## CPT alternates D_a = FALSE and D_a = TRUE between each row
    ## starting from D_a = FALSE
    cptfreqs <- as.numeric(rbind(seen.probs, unseen.probs))
    set.table(domain, D[a], cptfreqs, type = "cpt")
  }
  sapply(seq_along(D), one.allele)
  invisible(NULL)
}


