##' Set tables for auxiliary variables \code{O}.
##'
##' Function for setting the tables on all nodes \code{O_r_a} for
##' mixture \code{r} in the network corresponding to one marker, using
##' peak heights and a set of parameters. The returned evidence can be
##' used for conditioning on observed peak heights.
##' 
##' @details
##' The function sets probabilities in CPT according to whether a peak is observed
##' or not. If a peak is seen at allele \eqn{a}, the conditional distribution is defined using the p.d.f. \eqn{g} for the gamma distribution presented in \code{\link{DNAmixtures}} as
##' \deqn{P(\texttt{O[a]} = \texttt{TRUE} | n_{ia}, n_{i,a+1}, i = 1,\ldots, k) = g(\texttt{heights[a]})/k_a
##' }{P(O[a] = TRUE | n_{ia}, n_{i,a+1}, i = 1,\ldots, k) = g(heights[a])/k_a}
##' and likelihood-evidence \eqn{(0, k_a)} for the states (\code{FALSE}, \code{TRUE}) is returned.
##' If the allele is not seen, the CPT is set using the c.d.f. \eqn{G} for the gamma distribution as
##' \deqn{P(\code{O[a]} = \code{FALSE} | n_{ia}, n_{i,a+1}, i = 1,\ldots, k) = G(\code{C}),
##' }{P(O[a] = FALSE | n_{ia}, n_{i,a+1}, i = 1,\ldots, k) = G(C),}
##' in which case likelihood-evidence (1,0) is returned for this allele.
##'
##' @section Technical comments: For the sake of speed and saving
##' memory, only probabilities for the CPT of \code{O[a]} are
##' calculated, and not the entire table.  The parent nodes are
##' \code{n_i_a} and possibly \code{n_i_(a+1)} for all unknown
##' contributors i; the \eqn{n_{ia}} for known contributors are
##' specified through the argument \code{n_K}.  The layout of the
##' table relies in particular on the fact that all nodes have a
##' statespace {0,1,2}, which this is *not* checked anywhere. Thus, if
##' other types of allele-count-nodes \code{n_i_a} are introduced
##' (such as could be relevant for Amelogenin), the function should be
##' changed accordingly.
##'
##' If invalid tables are set then any subsequent propagation will
##' fail. No roll-back functionality has so far been implemented to
##' fix this, and the easiest solution is to re-fit the mixture model.
##'
##' Note that the detection threshold and model parameters can be
##' specified as vectors, and so it is in theory possible to use allele-specific
##' values, if desired. This also holds for the functions
##' \code{\link{setCPT.D}} and \code{\link{setCPT.Q}}.

##' @export
##' 
##' @param domain A \code{hugin.domain} modelling one marker
##' @param rho rho
##' @param xi xi
##' @param eta eta
##' @param phi phi
##' @param heights Peak heights
##' @param C Detection threshold
##' @param n.unknown Number of unknown contributors
##' @param n_K Allelecounts for known contributors
##' @param O Vector of names for binary nodes
##' @param gets_stutter Does the allele receive stutter? A boolean vector.
##' @param can_stutter Can the allele stutter?
##' @param stutter.from From which allele does the stutter come?
##' @return A matrix with each column containing the evidence to be set on \code{O[a]} when conditioning on the peak heights.
setCPT.O <- function(domain, rho, xi, eta, phi, heights, C, n.unknown,
                     n_K, O, gets_stutter, can_stutter, stutter.from){

  stopifnot(n.unknown > 0)
  
  ## Combinations of allele counts for all unknown contributors (at one allele)
  n_U <- expand.grid(rep(list(0:2), n.unknown), KEEP.OUT.ATTRS = FALSE)
  
  one.allele <- function(a){
    ## Calculate shape parameters
    
    ## Contribution from allele a
    n_a <- as.matrix(merge(n_U, n_K[a,])) ## = n_U if n_K is NULL
    shapes <- rho * (1 - xi*can_stutter[a]) * (n_a %*% phi)
    
    if (gets_stutter[a]){
      ## Adding contributions from stutter from allele b    
      b <- stutter.from[a]
      n_b <- as.matrix(merge(n_U, n_K[b,])) ## using n_Ua = n_Ub
      newshapes <- rho * xi * (n_b %*% phi)
      shapes <- outer(shapes, newshapes, "+")
    }
    shapes <- as.vector(shapes)

    ## If allele a is seen: P(O_a = TRUE) = f(height; allele counts)/Const
    if (heights[a] > 0){
      
      ## Gamma densities are scaled s.t. they are at most 1.
      seen.densities <- dgamma(heights[a], shape = shapes, scale = eta)
      Const <- max(seen.densities)
      
      ## CPT alternates O_a = FALSE and O_a = TRUE between each row
      ## starting from O_a = FALSE
      seen.probs <- seen.densities/Const
      cptfreqs <- as.numeric(rbind(1 - seen.probs, seen.probs))
      evidence <- c(0, Const)
    }
    ## If height = 0: P(O_a = FALSE) = P(Z_a < C)
    else {
      unseen.probs <- ifelse(shapes > 0,
                             pgamma(C, shape = shapes, scale = eta),
                             1)
      
      cptfreqs <- as.numeric(rbind(unseen.probs, 1 - unseen.probs))
      ## Evidence that O_a is FALSE
      evidence <- c(1, 0)
    }

    ## Keep previous table to revert to a stable CPT in case the
    ## following action fails? NOT IMPLEMENTED
    
    ## Set the CPT
    set.table(domain, O[a], cptfreqs, type = "cpt")

    ## Return evidence for future use in conditioning on peak heights
    evidence
  }

  ## Iterate over all alleles
  sapply(seq_along(O), one.allele)
}

